import contextlib

import pandas as pd
from sensospot_tools import (
    normalize as normalize_xdr_data,
)
from sensospot_tools import (
    select_hdr_data,
    split,
)

# column names

EXPOSURE_ID = "Exposure.Id"
EXPOSURE_CHANNEL = "Exposure.Channel"
EXPOSURE_TIME = "Exposure.Time"
EXPOSURE_TIME_NORMALIZED = "Exposure.Time.Normalized"
NET_SIGNAL = "NetSignal"
SPOT_SATURATION = "Spot.Saturation"
TEST_OVERFLOW = "Test.Spot.Overflow"

EXPOSURE_COLUMNS = [
    EXPOSURE_ID,
    EXPOSURE_CHANNEL,
    EXPOSURE_TIME,
    EXPOSURE_TIME_NORMALIZED,
]

SPOT_ID_COLUMNS = ["Analysis.Name", "Well.Name", "Pos.Id"]

TIME_DEPENDENT_COLUMNS = [
    "Bkg.Mean",
    "Bkg.Median",
    "Bkg.StdDev",
    "Bkg.Sum",
    "Spot.Mean",
    "Spot.Median",
    "Spot.StdDev",
    "Spot.Sum",
]

SATURATION_LIMIT = 2


def add_exposure_info(data: pd.DataFrame, analysis="hyb") -> pd.DataFrame:
    time_cy3 = 100 if "1" in analysis else 200
    exposure_values = [
        (1, "Cy3", time_cy3, time_cy3),
        (2, "Cy5", 150, 25),
        (3, "Cy5", 15, 25),
    ]
    exposure_df = pd.DataFrame(exposure_values, columns=EXPOSURE_COLUMNS)
    return data.merge(exposure_df, on=EXPOSURE_ID)


def test_overflow(data: pd.DataFrame, result_column: str = TEST_OVERFLOW):
    data[result_column] = data[SPOT_SATURATION] > SATURATION_LIMIT
    return data


def normalize(raw_data: pd.DataFrame, analysis="hyb"):
    with_exposure_info = add_exposure_info(raw_data, analysis=analysis)
    overflow_tested = test_overflow(with_exposure_info)
    normalized_data = (
        _normalize_channel(data)
        for _, data in split(overflow_tested, EXPOSURE_CHANNEL)
    )
    return pd.concat(normalized_data).reset_index(drop=True)


def _normalize_channel(data: pd.DataFrame) -> pd.DataFrame:
    normalized_time = data[EXPOSURE_TIME_NORMALIZED].unique()[0]
    hdr_data = select_hdr_data(
        data, SPOT_ID_COLUMNS, EXPOSURE_TIME, TEST_OVERFLOW
    )
    available_columns = [
        c for c in hdr_data.columns if c in TIME_DEPENDENT_COLUMNS
    ]
    normalized = normalize_xdr_data(
        hdr_data,
        normalized_time,
        EXPOSURE_TIME,
        available_columns,
        "{}.Normalized",
    )
    with contextlib.suppress(KeyError):
        normalized[NET_SIGNAL] = (
            normalized["Spot.Mean.Normalized"]
            - normalized["Bkg.Median.Normalized"]
        )

    return normalized
